@@ -5,6 +5,7 @@ import numpy as np
 import click
 from pathlib2 import Path
 import h5py
+import six
 
 from bluepysnap.config import Config
 
@@ -169,9 +170,14 @@ def _find_nodes_population(node_population_name, nodes):
     return None
 
 
-def _get_group_name(group):
-    """Gets group name of h5 group."""
-    return Path(group.parent.name).name
+def _get_group_name(group, parents=0):
+    """Gets group name of h5 group.
+
+    Args:
+        group (h5py.Group): nodes group in nodes .h5 file
+        parents (int): number of extra parents needed
+    """
+    return Path(*Path(group.name).parts[-(parents + 1):])
 
 
 def _get_h5_data(h5, path):
@@ -219,7 +225,7 @@ def _check_bio_nodes_group(group, config):
                                        format(group_name, group.file.filename)))
 
     errors = []
-    group_name = _get_group_name(group)
+    group_name = _get_group_name(group, parents=1)
     missing_fields = sorted({'morphology', 'x', 'y', 'z'} - set(group))
     if missing_fields:
         errors.append(fatal('Group {} of {} misses biophysical fields: {}'.
@@ -243,6 +249,18 @@ def _check_bio_nodes_group(group, config):
     return errors
 
 
+def _is_biophysical(group):
+    """Check if a group contains biophysical nodes."""
+    if group['model_type'][0] == 'biophysical':
+        return True
+    if "@library/model_type" in group:
+        model_type_int = group['model_type'][0]
+        model_type = group["@library/model_type"][model_type_int]
+        if six.ensure_str(model_type) == 'biophysical':
+            return True
+    return False
+
+
 def _check_nodes_group(group, config):
     """Validates nodes group in nodes population.
 
@@ -257,8 +275,9 @@ def _check_nodes_group(group, config):
     missing_fields = sorted(set(REQUIRED_GROUP_NAMES) - set(group))
     if missing_fields:
         return [fatal('Group {} of {} misses required fields: {}'
-                      .format(_get_group_name(group), group.file.filename, missing_fields))]
-    elif 'biophysical' in group['model_type'][:]:
+                      .format(_get_group_name(group, parents=1), group.file.filename,
+                              missing_fields))]
+    elif _is_biophysical(group):
         return _check_bio_nodes_group(group, config)
     return []
 
@@ -315,7 +334,8 @@ def _check_edges_group_bbp(group):
     missing_fields = sorted(set(GROUP_NAMES) - set(group))
     if missing_fields:
         return [BbpError(Error.WARNING, 'Group {} of {} misses fields: {}'.
-                         format(_get_group_name(group), group.file.filename, missing_fields))]
+                         format(_get_group_name(group, parents=1), group.file.filename,
+                                missing_fields))]
     return []
 
 
@@ -408,6 +428,80 @@ def _check_edges_indices(population):
     return errors
 
 
+def _get_edge_population_groups(population):
+    """Get groups from a edge population."""
+    return [name for name in population
+            if name != "indices" and isinstance(population[name], h5py.Group)]
+
+
+def _check_edge_population_data(population, groups, nodes):
+    # pylint: disable=too-many-locals,too-many-return-statements,too-many-branches
+    errors = []
+    population_name = _get_group_name(population)
+    if len(groups) > 1:
+        errors.append(BbpError(Error.WARNING, 'Population {} of {} have multiple groups. '
+                                              'Cannot be read via bluepysnap or libsonata'.
+                               format(population_name, population.file.filename)))
+
+    children_object_names = set(population)
+    group_datasets = ["edge_group_id", "edge_group_index"]
+    required_datasets = ['edge_type_id', 'source_node_id', 'target_node_id']
+    if len(groups) > 1:
+        required_datasets += group_datasets
+
+    missing_datasets = sorted(set(required_datasets) - children_object_names)
+    if missing_datasets:
+        return errors + [fatal('Population {} of {} misses datasets {}'.
+                               format(population_name, population.file.filename, missing_datasets))]
+    if len(groups) == 0:
+        return errors
+
+    # pure group part
+    missing_group_datasets = set(group_datasets) - set(population)
+    if len(missing_group_datasets) == 1:
+        return errors + [fatal('Population {} of {} misses dataset {}'.
+                               format(population_name, population.file.filename,
+                                      missing_group_datasets))]
+    if len(missing_group_datasets) == 2 and len(groups) == 1:
+        # no "edge_group_id", "edge_group_index" and only one group --> can use implicit ids
+        return errors
+
+    edge_group_ids = population["edge_group_id"][:]
+    edge_group_index = population["edge_group_index"][:]
+
+    if len(edge_group_ids) != len(edge_group_index):
+        return errors + [fatal('Population {} of {} "edge_group_id" and "edge_'
+                               'group_index" of different sizes'.
+                               format(population_name, population.file.filename))]
+
+    group_ids = np.unique(edge_group_ids)
+    missing_groups = set(group_ids) - set(np.array(groups, dtype=int))
+
+    if missing_groups:
+        return errors + [fatal('Population {} of {} misses group(s): {}'.
+                               format(population_name, population.file.filename, missing_groups))]
+    for group_id in group_ids:
+        group = population[str(group_id)]
+        max_edge_id = edge_group_index[edge_group_ids == int(group_id)].max()
+        if group[list(group)[0]].shape[0] < max_edge_id:
+            errors.append(fatal('Group {} in file {} should have ids up to {}'.
+                                format(_get_group_name(group, parents=1), population.file.filename,
+                                       max_edge_id)))
+
+    if 'source_node_id' in children_object_names:
+        errors += _check_edges_node_ids(population['source_node_id'], nodes)
+    if 'target_node_id' in children_object_names:
+        errors += _check_edges_node_ids(population['target_node_id'], nodes)
+    if 'indices' in children_object_names:
+        errors += _check_edges_indices(population)
+
+    for name in groups:
+        if isinstance(population[name], h5py.Group):
+            errors += _check_edges_group_bbp(population[name])
+
+    return errors
+
+
 def _check_edges_population(edges_dict, nodes):
     """Validates edges population.
 
@@ -418,8 +512,6 @@ def _check_edges_population(edges_dict, nodes):
     Returns:
         list: List of errors, empty if no errors
     """
-    POPULATION_DATASET_NAMES = [
-        'edge_type_id', 'source_node_id', 'target_node_id', 'edge_group_id', 'edge_group_index']
     errors = []
     edges_file = edges_dict.get('edges_file')
     with h5py.File(edges_file, 'r') as h5f:
@@ -427,24 +519,14 @@ def _check_edges_population(edges_dict, nodes):
         if not edges or len(edges) == 0:
             errors.append(fatal('No "edges" in {}.'.format(edges_file)))
             return errors
+
         for population_name in edges:
+
             population_path = '/edges/' + population_name
             population = h5f[population_path]
-            children_names = set(population.keys())
-            missing_datasets = sorted(set(POPULATION_DATASET_NAMES) - children_names)
-            if missing_datasets:
-                errors.append(fatal('Population {} of {} misses datasets {}'.
-                                    format(population_name, edges_file, missing_datasets)))
-                return errors
-            for name in children_names - {'indices'}:
-                if isinstance(population[name], h5py.Group):
-                    errors += _check_edges_group_bbp(population[name])
-            if 'source_node_id' in children_names:
-                errors += _check_edges_node_ids(population['source_node_id'], nodes)
-            if 'target_node_id' in children_names:
-                errors += _check_edges_node_ids(population['target_node_id'], nodes)
-            if 'indices' in children_names:
-                errors += _check_edges_indices(population)
+            groups = _get_edge_population_groups(population)
+            errors += _check_edge_population_data(population, groups, nodes)
+
     return errors
 
 
